#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Tue Aug  2 00:03:33 2022

@author: puneetgarg
"""

import numpy as np


from sphere_array import (
    get_A_inc,
    get_A_inc_comp_t,
    get_T_comp_t_n,
    get_C_sum_comp_t,
    get_RT,
)
from sphere_functions import S
from math_functions import get_modes
import matplotlib.pyplot as plt
import string
from time import perf_counter

t_i = perf_counter()


"""natural constants in SI units"""

e = 1.6 * 1e-19  # charge on an electron

m_e = 9.11 * 1e-31  # mass of an electron

eps_o = 8.854 * 1e-12  # free space permittivity

c_o = 3 * 1e8  # speed of light in vaccum


"""material parameters"""

c = 1.5 * 1e15  # dimensionless constant

wn = 4 * c  # resonance frequency

yn = wn / 8

No = (11 * wn ** 2 * eps_o * m_e) / (e ** 2)  # free electron density

R = 216 * 1e-9

"""time modulation parameters"""

wm = wn / 10

Ms = [0, 0.5, 0.9]

"""incident wave"""

winc = np.arange(0.0187 * wn, 1.012 * wn, wn / 1000)

ko = winc / c_o  # free space wavenumber

polpw = 0  # polarisation of incident wave

tinc = 0  # angle of incidencein yz plane

Egsd_i = np.array([1, 0])

"""floquet comb"""

jo = -4  # appropriately chosen integer

NOm = 8  # total number of frequencies in the floquet comb


"""Lattice parameters"""

a1 = 610

a = np.array([[a1 * 1e-9, 0], [0, a1 * 1e-9]])  # lattice vectors

rs = np.zeros(3)  # shift vector


"""Miscellaneous"""

l_max = 3  # maximum considered order of the multipole

modes = get_modes(l_max)


md = int(-jo * 2 * l_max * (l_max + 2))
ed = int(-jo * 2 * l_max * (l_max + 2) + l_max * (l_max + 2))

for M in range(0, len(Ms)):
    Ref = np.zeros(len(winc))  # reflectivity

    Tra = np.zeros(len(winc))

    Ted = np.zeros(len(winc))

    Tmd = np.zeros(len(winc))

    aTed = np.zeros(len(winc))

    aTmd = np.zeros(len(winc))

    for i in range(0, len(winc)):

        print(
            "iterations done:" + str(i + 1 + M * len(winc)) + "/" + str(3 * len(winc))
        )
        kpar = np.array([0, np.sin(tinc) * winc[i] / c_o])

        A_inc = get_A_inc(
            ko[i], kpar[0], kpar[1], np.cos(tinc) * winc[i] / c_o, polpw, l_max
        )  # incident field, only works for normal incidence

        A_inc_comp_t = get_A_inc_comp_t(jo, NOm, A_inc)

        k, S_k, k_o_ar = S(winc[i], NOm, jo, No, wn, yn, Ms[M], wm)

        T_comp_t = get_T_comp_t_n(l_max, NOm, k, S_k, k_o_ar, R)  # of single sphere

        C_sum_comp_t = get_C_sum_comp_t(modes, jo, wm, NOm, winc[i], kpar, a, rs, l_max)

        I = np.eye(len(T_comp_t))

        Teff = np.linalg.inv(I - (T_comp_t @ C_sum_comp_t)) @ T_comp_t  # of cluster

        A_sca = Teff @ A_inc_comp_t

        Tmd[i] = np.abs(A_sca[md])
        aTmd[i] = np.angle(A_sca[md])
        Ted[i] = np.abs(A_sca[ed])
        aTed[i] = np.angle(A_sca[ed])

        for j in range(0, NOm):

            wj = winc[i] + (
                (j + jo) * wm
            )  # frequency at which the scattered field is calculated

            if wj >= 0:

                Ref[i] += -get_RT(kpar, wj, j, ko[i], l_max, a, -1, A_sca, Egsd_i)
                Tra[i] += get_RT(kpar, wj, j, ko[i], l_max, a, 1, A_sca, Egsd_i)

    if M == 0:
        data1 = [Ref, Tra, Tmd / Ted, 180 * (aTmd - aTed) / np.pi]
    elif M == 1:
        data4 = [Ref, Tra, Tmd / Ted, 180 * (aTmd - aTed) / np.pi]
    elif M == 2:
        data9 = [Ref, Tra, Tmd / Ted, 180 * (aTmd - aTed) / np.pi]

# np.save('Ms_0.9.npy',[Ref, Tra, Tmd/Ted, 180*(aTmd-aTed)/np.pi])

print("generating plot...")

# plotting for paper###
a = a[0, 0]


fig, ax = plt.subplots(2, 3, figsize=(12, 8))
ax[1, 2].set_visible(False)
ax[1, 0].set_position([0.24, 0.125, 0.228, 0.343])
ax[1, 1].set_position([0.55, 0.125, 0.228, 0.343])


ax[0, 0].plot(winc / wn, data1[0], color="dodgerblue", linewidth=2)
ax[0, 0].plot(winc / wn, data4[0], color="tab:orange", linewidth=2)
ax[0, 0].plot(winc / wn, data9[0], color="magenta", linewidth=2)
ax[0, 0].set_xlim(min(winc) / wn, 1)
ax[0, 0].set_ylim(0, 1.03)
ax[0, 0].set_xlabel("$\omega_\mathrm{inc}/\omega_n$", fontsize=10)
ax[0, 0].set_ylabel("$\mathscr{R}$", rotation="horizontal", fontsize=12)
ax[0, 0].set_xticks([0, 0.5, 1])
ax[0, 0].set_xticklabels([0, 0.5, 1], fontsize=10)
ax[0, 0].set_yticks([0, 0.5, 1])
ax[0, 0].set_yticklabels([0, 0.5, 1], fontsize=10)
ax[0, 0].axvspan(
    min(winc) / wn, (2 * np.pi * c_o / a) / wn, color="blue", alpha=0.05, lw=0
)
ax[0, 0].set_title("Reflectivity", fontsize=10)
ax[0, 0].axvline(x=0.18, color="black", alpha=0.45, linestyle="-")
ax[0, 0].axvline(x=0.32, color="black", alpha=0.45, linestyle="-")
ax[0, 0].grid()

ax[0, 1].plot(winc / wn, data1[1], color="dodgerblue", linewidth=2)
ax[0, 1].plot(winc / wn, data4[1], color="tab:orange", linewidth=2)
ax[0, 1].plot(winc / wn, data9[1], color="magenta", linewidth=2)
ax[0, 1].set_xlim(min(winc) / wn, 1)
ax[0, 1].set_ylim(0, 1.03)
ax[0, 1].set_xlabel("$\omega_\mathrm{inc}/\omega_n$", fontsize=10)
ax[0, 1].set_ylabel("$\mathscr{T}$", rotation="horizontal", fontsize=12)
ax[0, 1].set_xticks([0, 0.5, 1])
ax[0, 1].set_xticklabels([0, 0.5, 1], fontsize=10)
ax[0, 1].set_yticks([0, 0.5, 1])
ax[0, 1].set_yticklabels([0, 0.5, 1], fontsize=10)
ax[0, 1].axvspan(
    min(winc) / wn, (2 * np.pi * c_o / a) / wn, color="blue", alpha=0.05, lw=0
)
ax[0, 1].set_title("Transmissivity", fontsize=10)
ax[0, 1].grid()

ax[0, 2].plot(winc / wn, 1 - data1[0] - data1[1], color="dodgerblue", linewidth=2)
ax[0, 2].plot(winc / wn, 1 - data4[0] - data4[1], color="tab:orange", linewidth=2)
ax[0, 2].plot(winc / wn, 1 - data9[0] - data9[1], color="magenta", linewidth=2)
ax[0, 2].set_xlim(min(winc) / wn, 1)
ax[0, 2].set_xlabel("$\omega_\mathrm{inc}/\omega_n$", fontsize=10)
ax[0, 2].set_ylabel("$\mathscr{A}$", rotation="horizontal", fontsize=12)
ax[0, 2].set_xticks([0, 0.5, 1])
ax[0, 2].set_xticklabels([0, 0.5, 1], fontsize=10)
ax[0, 2].set_yticks([0, 0.5, 1])
ax[0, 2].set_yticklabels([0, 0.5, 1], fontsize=10)
ax[0, 2].axvspan(
    min(winc) / wn, (2 * np.pi * c_o / a) / wn, color="blue", alpha=0.05, lw=0
)
ax[0, 2].set_title("Absorptivity", fontsize=10)
ax[0, 2].grid()

ax[1, 0].plot(winc / wn, data1[2], color="dodgerblue", linewidth=2)
ax[1, 0].plot(winc / wn, data4[2], color="tab:orange", linewidth=2)
ax[1, 0].plot(winc / wn, data9[2], color="magenta", linewidth=2)
ax[1, 0].set_xlim(min(winc) / wn, 1)
ax[1, 0].set_xlabel("$\omega_\mathrm{inc}/\omega_n$", fontsize=10)
ax[1, 0].set_ylabel("$A_\mathrm{MD}/A_\mathrm{ED}$", fontsize=10)
ax[1, 0].set_xticks([0, 0.5, 1])
ax[1, 0].set_xticklabels([0, 0.5, 1], fontsize=10)
ax[1, 0].set_yticks([0, 2.5, 5])
ax[1, 0].set_yticklabels([0, 2.5, 5], fontsize=10)
ax[1, 0].axvspan(
    min(winc) / wn, (2 * np.pi * c_o / a) / wn, color="blue", alpha=0.05, lw=0
)
ax[1, 0].axvline(x=0.18, color="black", alpha=0.45, linestyle="-")
ax[1, 0].axvline(x=0.32, color="black", alpha=0.45, linestyle="-")
ax[1, 0].axhline(y=1, color="black", alpha=0.45, linestyle="-")
ax[1, 0].grid()

ax[1, 1].plot(winc / wn, data1[3], color="dodgerblue", linewidth=2)
ax[1, 1].plot(winc / wn, data4[3], color="tab:orange", linewidth=2)
ax[1, 1].plot(winc / wn, data9[3], color="magenta", linewidth=2)
ax[1, 1].set_xlim(min(winc) / wn, 1)
ax[1, 1].set_ylim(-360, 0)
ax[1, 1].set_xlabel("$\omega_\mathrm{inc}/\omega_n$", fontsize=10)
ax[1, 1].set_ylabel(r"$\angle{A_\mathrm{MD}}-\angle{A_\mathrm{ED}}$", fontsize=10)
ax[1, 1].set_xticks([0, 0.5, 1])
ax[1, 1].set_xticklabels([0, 0.5, 1], fontsize=10)
ax[1, 1].set_yticks([0, -90, -180, -270, -360])
ax[1, 1].set_yticklabels(
    ["$0$", "$-\pi/2$", "$-\pi$", "$-3\pi/2$", "$-2\pi$"], fontsize=10
)
ax[1, 1].axvspan(
    min(winc) / wn, (2 * np.pi * c_o / a) / wn, color="blue", alpha=0.05, lw=0
)
ax[1, 1].axvline(x=0.18, color="black", alpha=0.45, linestyle="-")
ax[1, 1].axvline(x=0.32, color="black", alpha=0.45, linestyle="-")
ax[1, 1].axhline(y=-180, color="black", alpha=0.45, linestyle="-")
ax[1, 1].grid()


line_labels = ["$M_\mathrm{s}=0$", "$M_\mathrm{s}=0.5$", "$M_\mathrm{s}=0.9$"]
fig.legend(
    labels=line_labels,  # The labels for each line
    loc="right",  # Position of legend
    borderaxespad=0.05,  # Small spacing around legend box.
    fancybox=True,
    bbox_to_anchor=(0.9, 0.4),
    fontsize=10,
)

ax = ax.flat
for n, axs in enumerate(ax):

    axs.text(
        0.85, 0.85, string.ascii_lowercase[n] + ")", transform=axs.transAxes, size=10
    )
